/*
 
 File: CanvasDoc.h
 
 Abstract: Document model object
 
 Version: 1.0
 
 © Copyright 2004 Apple Computer, Inc. All rights reserved.
 
 IMPORTANT:  This Apple software is supplied to 
 you by Apple Computer, Inc. ("Apple") in 
 consideration of your agreement to the following 
 terms, and your use, installation, modification 
 or redistribution of this Apple software 
 constitutes acceptance of these terms.  If you do 
 not agree with these terms, please do not use, 
 install, modify or redistribute this Apple 
 software.
 
 In consideration of your agreement to abide by 
 the following terms, and subject to these terms, 
 Apple grants you a personal, non-exclusive 
 license, under Apple's copyrights in this 
 original Apple software (the "Apple Software"), 
 to use, reproduce, modify and redistribute the 
 Apple Software, with or without modifications, in 
 source and/or binary forms; provided that if you 
 redistribute the Apple Software in its entirety 
 and without modifications, you must retain this 
 notice and the following text and disclaimers in 
 all such redistributions of the Apple Software. 
 Neither the name, trademarks, service marks or 
 logos of Apple Computer, Inc. may be used to 
 endorse or promote products derived from the 
 Apple Software without specific prior written 
 permission from Apple.  Except as expressly 
 stated in this notice, no other rights or 
 licenses, express or implied, are granted by 
 Apple herein, including but not limited to any 
 patent rights that may be infringed by your 
 derivative works or by other works in which the 
 Apple Software may be incorporated.
 
 The Apple Software is provided by Apple on an "AS 
 IS" basis.  APPLE MAKES NO WARRANTIES, EXPRESS OR 
 IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED 
 WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY 
 AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING 
 THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE 
 OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL APPLE BE LIABLE FOR ANY 
 SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL 
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
 OF USE, DATA, OR PROFITS; OR BUSINESS 
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, 
 REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF 
 THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER 
 UNDER THEORY OF CONTRACT, TORT (INCLUDING 
 NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN 
 IF APPLE HAS BEEN ADVISED OF THE POSSIBILITY OF 
 SUCH DAMAGE.
 
 */ 

#import <Cocoa/Cocoa.h>
#import "CanvasView.h"
#import "CanvasObject.h"
#import "CanvasManagedUndoController.h"

@interface CanvasDoc : NSDocument
{
	// Document attributes
	NSColor *backgroundColor;
	NSMutableArray *canvasObjects;
	NSSize size;
	
	// Canvas object selection
	NSMutableSet *selectedObjects;
	
	// Undo management
	CanvasManagedUndoController *undoController;

	// Outlets
	IBOutlet CanvasView *canvas;
	IBOutlet NSArrayController *objectController;
	
	// Document state
	BOOL isDocumentClosed;
}

#pragma mark NSDocument methods
- (void)close;
- (NSData *)dataRepresentationOfType:(NSString *)aType;
- (BOOL)loadDataRepresentation:(NSData *)data ofType:(NSString *)aType;
- (void)windowControllerDidLoadNib:(NSWindowController *)aController;
- (NSString *)windowNibName;

#pragma mark Canvas object accessors
- (NSArray *)canvasObjects;
- (int)countOfCanvasObjects;
- (id)objectInCanvasObjectsAtIndex:(int)index;
- (void)addObjectInCanvasObjects:(id)anObject;
- (void)insertObject:(id)anObject inCanvasObjectsAtIndex:(int)index;
- (void)removeObjectFromCanvasObjects:(id)anObject;
- (void)removeObjectFromCanvasObjectsAtIndex:(int)index;
- (void)replaceObjectInCanvasObjectsAtIndex:(int)index withObject:(id)anObject;

#pragma mark Canvas object ordering
- (void)moveCanvasObjectForward:(CanvasObject *)anObject;
- (void)moveCanvasObjectBackward:(CanvasObject *)anObject;

#pragma mark Canvas object selection
- (NSSet *)selectedObjects;
- (NSArray *)selectedObjectsArray;
- (void)setSelectedObjects:(NSSet *)aSelectionSet;
- (void)addCanvasObjectsToSelection:(NSArray *)canvasObjects;
- (void)removeCanvasObjectsFromSelection:(NSArray *)canvasObjects;
- (void)selectCanvasObject:(CanvasObject *)canvasObject;

#pragma mark Document attribute accessors
- (NSColor *)backgroundColor;
- (void)setBackgroundColor:(NSColor *)aColor;
- (NSSize)size;
- (void)setSize:(NSSize)aSize;
- (float)sizeWidth;
- (void)setSizeWidth:(float)aFloat;
- (float)sizeHeight;
- (void)setSizeHeight:(float)aFloat;

#pragma mark Managed undo support
+ (NSArray *)undoableKeys;
- (void)setValue:(id)aValue forUndoableKeyPath:(NSString *)keyPath;
- (void)insertObjects:(NSArray *)anArray intoUndoableKeyPath:(NSString *)keyPath;
- (void)removeObjects:(NSArray *)anArray fromUndoableKeyPath:(NSString *)keyPath;
- (void)replaceObjects:(NSArray *)oldObjects withObjects:(NSArray *)newObjects inUndoableKeyPath:(NSString *)keyPath;

@end

#pragma mark Comparator functions
extern int CanvasObjectSortByZOrder(CanvasObjectView *firstView, CanvasObjectView *secondView, void *context);
