/*
	File:		AVCBrowser/Plug.m

	Copyright: 	 Copyright 2001-2002 Apple Computer, Inc. All rights reserved.

	Written by: wgulland
	
	Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Computer, Inc.
				("Apple") in consideration of your agreement to the following terms, and your
				use, installation, modification or redistribution of this Apple software
				constitutes acceptance of these terms.  If you do not agree with these terms,
				please do not use, install, modify or redistribute this Apple software.

				In consideration of your agreement to abide by the following terms, and subject
				to these terms, Apple grants you a personal, non-exclusive license, under Apples
				copyrights in this original Apple software (the "Apple Software"), to use,
				reproduce, modify and redistribute the Apple Software, with or without
				modifications, in source and/or binary forms; provided that if you redistribute
				the Apple Software in its entirety and without modifications, you must retain
				this notice and the following text and disclaimers in all such redistributions of
				the Apple Software.  Neither the name, trademarks, service marks or logos of
				Apple Computer, Inc. may be used to endorse or promote products derived from the
				Apple Software without specific prior written permission from Apple.  Except as
				expressly stated in this notice, no other rights or licenses, express or implied,
				are granted by Apple herein, including but not limited to any patent rights that
				may be infringed by your derivative works or by other works in which the Apple
				Software may be incorporated.

				The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
				WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
				WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
				PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
				COMBINATION WITH YOUR PRODUCTS.

				IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
				CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
				GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
				ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
				OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
				(INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
				ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	$Log: Plug.mm,v $
	Revision 1.1  2003/03/20 22:48:10  firewire
	No log message.
	
	Revision 1.2  2002/11/08 22:46:48  firewire
	first checkin of install project
	
*/

#import "Plug.h"

@implementation Plug
-(Plug *)initWithDevice:(AVCDevice *)device index:(int)i val:(UInt32)v
{
    _device = [device retain];
    [self setIndex:[NSNumber numberWithUnsignedInt:i]];
    [self setOnline:[NSNumber numberWithUnsignedInt:(v >> 31) & 1]];
    [self setBroadcast:[NSNumber numberWithUnsignedInt:(v >> 30) & 1]];
    [self setPoint:[NSNumber numberWithUnsignedInt:(v >> 24) & 63]];
    [self setChannel:[NSNumber numberWithUnsignedInt:(v >> 16) & 63]];
    return self;
}

-(NSNumber *)index
{
    return _index;
}

-(void) setIndex:(NSNumber *)val
{
    [val retain];
    [_index release];
    _index = val;
}

-(NSNumber *)online
{
    return _online;
}

-(void) setOnline:(NSNumber *)val
{
    [val retain];
    [_online release];
    _online= val;
}

-(NSNumber *)broadcast
{
    return _broadcast;
}

-(void) setBroadcast:(NSNumber *)val
{
    [val retain];
    [_broadcast release];
    _broadcast = val;
}

-(NSNumber *)point
{
    return _point;
}

-(void) setPoint:(NSNumber *)val
{
    [val retain];
    [_point release];
    _point = val;
}

-(NSNumber *)channel
{
    return _channel;
}

-(void) setChannel:(NSNumber *)val
{
    [val retain];
    [_channel release];
    _channel = val;
}


@end

@implementation InputPlug
+(InputPlug *)withDevice:(AVCDevice *)device index:(int)i;
{
    FWAddress addr;
    UInt32 v;
    InputPlug *plug = [self alloc];
    addr.addressHi = 0xffff;
    addr.addressLo = 0xf0000984 + i*4;
    v = [device readQuad:addr];
    [plug initWithDevice: device index:i val:v];
    return plug;
}

@end

@implementation OutputPlug
+(OutputPlug *)withDevice:(AVCDevice *)device index:(int)i;
{
    FWAddress addr;
    UInt32 v;
    OutputPlug *plug = [self alloc];
    addr.addressHi = 0xffff;
    addr.addressLo = 0xf0000904 + i*4;
    v = [device readQuad:addr];
    [plug initWithDevice: device index:i val:v];
    [plug setRate:[NSNumber numberWithUnsignedInt:(v >> 14) & 3]];
    [plug setOverhead:[NSNumber numberWithUnsignedInt:(v >> 10) & 15]];
    [plug setPayload:[NSNumber numberWithUnsignedInt:v & 1023]];
    return plug;
}

-(NSNumber *)rate
{
    return _rate;
}

-(void) setRate:(NSNumber *)val
{
    [val retain];
    [_rate release];
    _rate = val;
}

-(NSNumber *)overhead
{
    return _overhead;
}

-(void) setOverhead:(NSNumber *)val
{
    [val retain];
    [_overhead release];
    _overhead = val;
}

-(NSNumber *)payload
{
    return _payload;
}

-(void) setPayload:(NSNumber *)val
{
    [val retain];
    [_payload release];
    _payload = val;
}


@end

