/*
	Created by: nwg
	
	Copyright: 	 Copyright 2002-2003 Apple Computer, Inc. All rights reserved.
	
	Disclaimer:	IMPORTANT:  This Apple software is supplied to you by Apple Computer, Inc.
				("Apple") in consideration of your agreement to the following terms, and your
				use, installation, modification or redistribution of this Apple software
				constitutes acceptance of these terms.  If you do not agree with these terms,
				please do not use, install, modify or redistribute this Apple software.

				In consideration of your agreement to abide by the following terms, and subject
				to these terms, Apple grants you a personal, non-exclusive license, under Apples
				copyrights in this original Apple software (the "Apple Software"), to use,
				reproduce, modify and redistribute the Apple Software, with or without
				modifications, in source and/or binary forms; provided that if you redistribute
				the Apple Software in its entirety and without modifications, you must retain
				this notice and the following text and disclaimers in all such redistributions of
				the Apple Software.  Neither the name, trademarks, service marks or logos of
				Apple Computer, Inc. may be used to endorse or promote products derived from the
				Apple Software without specific prior written permission from Apple.  Except as
				expressly stated in this notice, no other rights or licenses, express or implied,
				are granted by Apple herein, including but not limited to any patent rights that
				may be infringed by your derivative works or by other works in which the Apple
				Software may be incorporated.

				The Apple Software is provided by Apple on an "AS IS" basis.  APPLE MAKES NO
				WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED
				WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A PARTICULAR
				PURPOSE, REGARDING THE APPLE SOFTWARE OR ITS USE AND OPERATION ALONE OR IN
				COMBINATION WITH YOUR PRODUCTS.

				IN NO EVENT SHALL APPLE BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR
				CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
				GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
				ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION
				OF THE APPLE SOFTWARE, HOWEVER CAUSED AND WHETHER UNDER THEORY OF CONTRACT, TORT
				(INCLUDING NEGLIGENCE), STRICT LIABILITY OR OTHERWISE, EVEN IF APPLE HAS BEEN
				ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

	$Log: NSDataTableDataSource.mm,v $
	Revision 1.2  2003/05/27 17:47:00  firewire
	SDK16
	
	Revision 1.1  2002/11/07 00:36:20  noggin
	move to new repository
	
	Revision 1.2  2002/08/06 22:23:51  noggin
	we were creating extra retains on services.. now we release them properly.
	
	Revision 1.1  2002/07/16 22:07:13  noggin
	now have NSData view and new icon
	
*/

#import "NSDataTableDataSource.h"


@implementation NSDataTableDataSource

- (void)awakeFromNib
{
	mData = nil ;
}

- (void)setData:(NSData*)data
{
	[ mData release ] ;
	mData = [ data retain ] ;
}

- (void)dealloc
{
	[ mData release ] ;
	[ super dealloc ] ;
}

// NSTableView data source
- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
	if (!mData)
		return 0 ;
		
	return [ mData length ] / 8 + ( ([ mData length ] % 8) > 0 ? 1 : 0 ) ;
}

- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(int)row
{
	if (!mData)
		return nil ;
		
	unsigned start = 8 * row ;
	unsigned length = [ mData length ] - start ;
	
	if ( length > 8 )
		length = 8 ;

	NSMutableString*	result = [ NSMutableString stringWithCapacity:19 ] ;
	const char* bytes = (const char*)[ mData bytes ] ;
	
	if ( [ [ tableColumn identifier ] isEqual:@"Offset" ] )
	{
		result = [ NSString stringWithFormat:@"%08X -  ", start ] ;
	}
	else if ( [ [ tableColumn identifier ] isEqual:@"Hex" ] )
	{
		for( unsigned index=0; index < length; ++index )
		{
			[ result appendFormat:@"%02lX ", (unsigned long)(bytes[start+index]) & 0xFF ] ;
			if ( index == 3 )
				[ result appendString:@"  " ] ;
		}
	}
	else
	{
		for( unsigned index=0; index < length; ++index )
		{
			if ( [ [ NSCharacterSet controlCharacterSet ] characterIsMember:(unichar)bytes[start + index] ] )
				[ result appendString:@"." ] ;
			else
				[ result appendFormat:@"%c", bytes[start + index] ] ;
		}
	}
	
	return result ;
}

@end
